use bevy::{
    prelude::*,
    sprite::{MaterialMesh2dBundle, Mesh2dHandle},
};
use bevy_rapier2d::prelude::*;
use drops::spawner_plugin;
use player::LifeChangeEvent;
const METER: f32 = 48.;

fn main() {
    let mut rapier_config = RapierConfiguration::new(METER * 2.5);
    rapier_config.timestep_mode = TimestepMode::Variable {
        max_dt: 1.0 / 60.0,
        time_scale: 1.0,
        substeps: 4,
    };
    App::new()
        .add_plugins(DefaultPlugins)
        // this does things with gravity
        .insert_resource(rapier_config)
        .add_plugins(RapierPhysicsPlugin::<NoUserData>::pixels_per_meter(METER))
        .add_plugins(RapierDebugRenderPlugin::default())
        .add_plugins((
            spawner_plugin,
            game_state::state_and_ui_plugin,
            player::player_plugin,
        ))
        .add_systems(Startup, (scene::setup_scene, setup_cam))
        .run()
}

fn setup_cam(mut commands: Commands) {
    commands.spawn(Camera2dBundle {
        transform: Transform::from_xyz(0., 4. * METER, 0.),
        ..default()
    });
}

mod drops;
mod player;
mod scene;
mod game_state {
    use bevy::{prelude::*, time::Stopwatch};

    #[derive(Component)]
    pub struct GameState {
        score: u32,
        game_time: Stopwatch,
    }

    #[derive(Component)]
    struct StateText;

    pub fn state_and_ui_plugin(app: &mut App) {
        app.add_systems(Startup, (setup_gamestate, setup_ui))
            .add_systems(Update, (update_time));
    }

    fn setup_gamestate(mut commands: Commands) {
        commands.spawn(GameState {
            score: 0,
            game_time: Stopwatch::new(),
        });
    }

    fn setup_ui(mut commands: Commands) {
        commands.spawn((
            TextBundle::from_sections([
                TextSection::new("Score: ", TextStyle::default()),
                TextSection::new("0", TextStyle::default()),
                TextSection::new("\n", TextStyle::default()),
                TextSection::new("Time: ", TextStyle::default()),
                TextSection::new("0", TextStyle::default()),
                TextSection::new("s", TextStyle::default()),
            ])
            .with_style(Style {
                position_type: PositionType::Absolute,
                top: Val::Px(5.0),
                right: Val::Px(5.0),
                ..default()
            }),
            StateText,
        ));
    }

    fn update_time(
        mut state_txt: Query<&mut Text, With<StateText>>,
        mut game_state: Query<&mut GameState>,
        time: Res<Time>,
    ) {
        let mut txt = state_txt.single_mut();
        let time = game_state
            .single_mut()
            .game_time
            .tick(time.delta())
            .elapsed_secs();

        txt.sections[4].value = format!("{time:.2}");
    }
}
